//    Copyright 2025 Bill Rossi
//    
//    This file is part of Starship Futuretime
//
//    Starship Futuretime is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//
//    Starship Futuretime is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License along with Starship Futuretime. If not, see <https://www.gnu.org/licenses/>. 
#ifndef PLAYER_H
#define PLAYER_H

#include <raylib.h>

#define ACCEL_X 1.7
#define MAX_VEL_X 35
#define DAMPING_X 0.85
#define MAX_VEL_Y 20
#define MIN_VEL_Y 1

#define MAX(a, b) ((a > b) ? a : b)
#define MIN(a, b) ((a < b) ? a : b)
#define CLAMP(a, b) (MIN(MAX(a, -b), b))

typedef enum PlayerPitch {
  PITCH_DESCENDING,
  PITCH_NEUTRAL,
  PITCH_ASCENDING,
} PlayerPitch;

typedef enum PlayerRoll {
  ROLL_LEFT,
  ROLL_NEUTRAL,
  ROLL_RIGHT,
} PlayerRoll;

typedef struct Player Player;

#include "game.h"

struct Player {
  Vector2 position;
  Vector2 velocity;
  Vector2 acceleration;
  PlayerPitch pitch;
  PlayerRoll roll;
  Texture2D spritesheet;
  Game *game;
};

void initialize_player(Game *g, Player *p);
void handle_player_input(Player *p);
void draw_player(Player *p);
void move_player(Player *p);

#endif
